package com.perforce.hws.server.routes.perforce;

import com.perforce.hws.p4base.*;
import com.perforce.hws.server.HWSSettings;
import com.perforce.hws.server.ModelConversion;
import com.perforce.hws.server.UsesServerHandles;
import com.perforce.hws.server.routes.UsesJsonRequests;
import com.perforce.hws.server.sessions.SessionData;
import com.perforce.hwsclient.models.LabelCommand;

import spark.Request;
import spark.Response;
import spark.Route;

import java.util.List;
import java.util.Map;
import java.util.function.Supplier;

/**
 * PATCH /p4/:api/labels/:label
 */
public class PerforceUpdateLabelRoute implements
        Route,
        P4Methods,
        ResultMapMethods,
        ModelConversion,
        HasGson,
        UsesJsonRequests,
        UsesServerHandles {

    @Override
    public Object handle(Request request, Response response) throws Exception {
        SessionData sessionData = request.attribute("sessionData");
        HWSSettings settings = request.attribute("settings");

        String server = request.params("server");
        String label = request.params("label");

        LabelCommand labelCommand = readModel(request, LabelCommand.class);

        Map<String, Object> updates = toRequestMap(labelCommand);

        return withServerHandle(server, sessionData, settings, serverHandle -> {

            List<ResultMap> results = exec(serverHandle, "label", "-o", label);
            ResultMap resultMap = firstResultOrFail(results);

            Supplier<String> offsetSupplier = offsetSupplier(serverHandle);

            Map<String, Object> current = toRequestMap(toLabelCommand(resultMap, offsetSupplier));

            current.putAll(updates);

            return toCommandResponse(execInput(serverHandle, current, "label", "-i"));
        });
    }
}
